package nl.kallestruik.discordbot

import dev.kord.common.entity.ButtonStyle
import dev.kord.common.entity.Snowflake
import dev.kord.core.Kord
import dev.kord.core.behavior.channel.createMessage
import dev.kord.core.behavior.interaction.response.respond
import dev.kord.core.builder.components.emoji
import dev.kord.core.entity.channel.TextChannel
import dev.kord.core.event.interaction.GuildButtonInteractionCreateEvent
import dev.kord.core.on
import dev.kord.rest.builder.message.create.actionRow
import kotlinx.coroutines.flow.*
import kotlinx.serialization.decodeFromString
import kotlinx.serialization.json.Json
import nl.kallestruik.discordbot.config.Config
import java.io.File

suspend fun main() {
    val config: Config = Json.decodeFromString(File("config.json").readText())

    val kord = Kord(config.token)

    val channel = kord.getChannel(Snowflake(config.roleChannelID)) as? TextChannel ?: return

    val oldMessages = mutableSetOf<Snowflake>()
    channel.messages
        .filter { it.author == kord.getSelf() }
        .map { it.id }
        .toSet(oldMessages)
    channel.bulkDelete(oldMessages)

    channel.createMessage {
        actionRow {
            for (role in config.roles) {
                val roleConfig = role.value

                interactionButton(ButtonStyle.Secondary, role.key) {
                    label = roleConfig.name
                    emoji(kord.getGuild(Snowflake(config.guildID))?.getEmoji(Snowflake(roleConfig.emojiID)) ?: return)
                }
            }
        }
    }

    kord.on<GuildButtonInteractionCreateEvent> {
        val response = interaction.deferEphemeralResponse()
        val roleConfig = config.roles[interaction.data.data.customId.value]
        if (roleConfig == null) {
            response.respond {
                content = "An error occurred while toggling your role!"
            }
            return@on
        }

        val roleId = Snowflake(roleConfig.id)

        if (interaction.user.roles.filter { it.id == roleId }.firstOrNull() == null) {
            interaction.user.addRole(roleId, reason = "Self assigned by user")
            response.respond {
                content = "Added you to role ${roleConfig.name}"
            }
        } else {
            interaction.user.removeRole(roleId, reason = "Self removed by user")
            response.respond {
                content = "Removed you from role ${roleConfig.name}"
            }
        }
    }

    kord.login()
}